use std::sync::Arc;

use chrono::Utc;
use futures_util::lock::Mutex;
use hmac::{Hmac, Mac};
use lazy_static::lazy_static;
use serde::Deserialize;
use sha2::Sha256;

use crate::{
    config,
    res::response::ResErr,
};

use super::SocialResult;

lazy_static! {
    pub static ref WECHAT_SOCIAL: WechatSocial = WechatSocial::default();
}

#[derive(Debug, Clone)]
pub struct WechatSocial {
    app_id: String,
    app_secret: String,
    access_token: Arc<Mutex<WeChatAccessToken>>,
}

#[derive(Deserialize, Debug, Clone, Default)]
pub struct WeChatAccessToken {
    pub access_token: String,
    pub expires_in: i64,
}

#[derive(Deserialize, Debug, Clone)]
pub struct WeChatBaseResult {
    // 错误码
    pub errcode: i64,
    // 错误信息
    pub errmsg: String,
}

#[derive(Deserialize, Debug, Clone)]
pub struct MiniAppLoginResult {
    // 用户唯一标识
    pub openid: String,
    // 会话密钥
    pub session_key: String,
    // 用户在开放平台的唯一标识符，若当前小程序已绑定到微信开放平台账号下会返回
    // 如果开发者拥有多个移动应用、网站应用、和公众账号（包括小程序），可通过 UnionID 来区分用户的唯一性
    // https://developers.weixin.qq.com/miniprogram/dev/framework/open-ability/union-id.html
    pub unionid: String,
    // 错误码
    pub errcode: i64,
    // 错误信息
    pub errmsg: String,
}

impl Default for WechatSocial {
    fn default() -> Self {
        let wechat_option = &config!().social.wechat;
        Self {
            app_id: wechat_option.app_id.to_owned(),
            app_secret: wechat_option.app_secret.to_owned(),
            access_token: Arc::new(Mutex::new(WeChatAccessToken {
                access_token: "".to_string(),
                expires_in: 0,
            })),
        }
    }
}

impl WechatSocial {
    /// 异步获取微信公钥，适用于小程序和小游戏
    /// 本接口和getStableAccessToken类似
    /// 区别为：
    ///     getAccessToken，每次调用都会生成新的access_token；
    ///     getStableAccessToken，在force_refresh=false模式下，会尽量返回相同的access_token。
    /// https://developers.weixin.qq.com/miniprogram/dev/OpenApiDoc/mp-access-token/getAccessToken.html
    /// 
    /// https://developers.weixin.qq.com/minigame/dev/api-backend/open-api/access-token/auth.getAccessToken.html
    /// https://developers.weixin.qq.com/miniprogram/dev/OpenApiDoc/mp-access-token/getAccessToken.html
    async fn fetch_and_parse_wechat_access_token(&self) -> SocialResult<WeChatAccessToken> {
        let mut wechat_access_token = self.access_token.lock().await;
        if wechat_access_token.access_token.is_empty()
            && wechat_access_token.expires_in < Utc::now().timestamp()
        {
            let response = reqwest::get(format!(
                "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid={}&secret={}",
                self.app_id,
                self.app_secret
            ))
            .await
            .unwrap();
            let mut new_wechat_access_token: WeChatAccessToken = response.json().await.unwrap();
            tracing::info!("wechat access token: {:?}", new_wechat_access_token);
            new_wechat_access_token.expires_in =
                Utc::now().timestamp() + wechat_access_token.expires_in;
            *wechat_access_token = new_wechat_access_token;
        }
        tracing::info!("wechat access token: {:?}", wechat_access_token);
        Ok(wechat_access_token.clone())
    }

    /// 微信登录登录
    /// 本接口适用于小程序和小游戏
    /// https://developers.weixin.qq.com/minigame/dev/api-backend/open-api/login/auth.code2Session.html
    /// https://developers.weixin.qq.com/miniprogram/dev/OpenApiDoc/user-login/code2Session.html
    pub async fn code_2_session(&self, code: &str) -> SocialResult<MiniAppLoginResult> {
        let response = reqwest::get(format!(
            "https://api.weixin.qq.com/sns/jscode2session?appid={}&secret={}&js_code={}&grant_type=authorization_code",
            self.app_id,
            self.app_secret,
            code
        ))
        .await
        .unwrap();
        let result: MiniAppLoginResult = response.json().await.unwrap();
        if result.errcode != 0 {
            tracing::error!(
                "微信登录失败,errcode:{},errmsg:{}",
                result.errcode,
                result.errmsg
            );
            return Err(Box::new(ResErr::social(format!(
                "微信登录失败,errcode:{},errmsg:{}",
                result.errcode, result.errmsg
            ))));
        }
        Ok(result)
    }

    /// 验证登录信息
    /// 本接口适用于小程序和小游戏
    /// 
    /// https://developers.weixin.qq.com/minigame/dev/api-backend/open-api/login/auth.checkSessionKey.html
    /// https://developers.weixin.qq.com/miniprogram/dev/OpenApiDoc/user-login/checkSessionKey.html
    /// todo 使用的hmac签名是否正确，签名结果转换成字符串是否正确，目前未经验证
    pub async fn check_session(&self, session_key: &str, open_id: &str) -> SocialResult<()> {
        // let signature = Hmac::<Sha256>::new_from_slice(session.as_bytes()).expect("");
        let mut mac =
            Hmac::<Sha256>::new_from_slice(session_key.as_bytes()).expect("初始化hmac失败");
        mac.update(b"");
        let out = mac.finalize().into_bytes();
        let signature = std::str::from_utf8(out.trim_ascii())
            .unwrap_or_default()
            .trim_ascii()
            .to_string();
        let response = reqwest::get(format!(
            "https://api.weixin.qq.com/wxa/checksession?access_token={}&signature={}&openid={}&sig_method=hmac_sha256",
            self.access_token.lock().await.access_token,
            signature,
            open_id
        ))
        .await
        .unwrap();
        let result: WeChatBaseResult = response.json().await.unwrap();
        if result.errcode != 0 {
            tracing::error!("微信登录校验失败,errcode:{},errmsg:{}", result.errcode, result.errmsg);
            return Err(Box::new(ResErr::social(format!(
                "微信登录校验失败,errcode:{},errmsg:{}",
                result.errcode, result.errmsg
            ))));
        }
        Ok(())
    }
}
